from plone.app.layout.viewlets import ViewletBase
from Products.Archetypes.interfaces import IBaseObject
from Products.CMFCore.utils import getToolByName
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile


class TopicReferences(ViewletBase):
    """Shows all topic references"""

    render = ViewPageTemplateFile('topic_references.pt')

    def get_references(self):
        result = []
        mtool = getToolByName(self.context, 'portal_membership')
        refs = sorted(self.context.Schema()['topics'].get(self.context),
                      key=lambda item: item.title_or_id())
        for obj in refs:
            if mtool.checkPermission('View', obj):
                result.append(dict(title=obj.title_or_id(),
                                   description=obj.Description(),
                                   url=obj.absolute_url(), ))
        return result

    def available(self):
        # XXX Only Archtypes support, implement DX support
        if not IBaseObject.providedBy(self.context):
            return False
        return bool(self.get_references())



from Products.CMFPlone.interfaces.siteroot import IPloneSiteRoot
from ftw.testing import MockTestCase
from ftw.topics.finder import DefaultTopicTreeFinder
from ftw.topics.interfaces import ITopicRootFinder
from ftw.topics.interfaces import ITopicTree
from ftw.topics.testing import ZCML_LAYER
from plone.app.layout.navigation.interfaces import INavigationRoot
from zope.component import getMultiAdapter
from zope.component import getSiteManager
from zope.component import queryMultiAdapter
from zope.component.hooks import setSite
from zope.interface.verify import verifyClass


class TestDefaultTopicTreeFinder(MockTestCase):

    layer = ZCML_LAYER

    def setUp(self):
        super(TestDefaultTopicTreeFinder, self).setUp()
        self.request = self.stub_request()

    def tearDown(self):
        super(TestDefaultTopicTreeFinder, self).tearDown()
        setSite(None)

    def create_tree(self, with_trees=True):
        objs = {}

        objs['site'] = self.providing_stub(IPloneSiteRoot, INavigationRoot)
        self.expect(objs['site'].getPhysicalPath()).result(['', 'site'])
        self.expect(objs['site'].getSiteManager).result(getSiteManager)

        if with_trees:
            objs['root'] = self.providing_stub(ITopicRootFinder)
            self.expect(objs['site'].objectValues()).result([
                    objs['root']])
        else:
            self.expect(objs['site'].objectValues()).result([])

        objs['foo'] = self.stub()
        self.expect(objs['foo'].getPhysicalPath()).result(['', 'site', 'foo'])
        self.set_parent(objs['foo'], objs['site'])

        objs['subsite'] = self.providing_stub(INavigationRoot)
        self.expect(objs['subsite'].getPhysicalPath()).result(
            ['', 'site', 'subsite'])
        self.set_parent(objs['subsite'], objs['site'])

        objs['bar'] = self.stub()
        self.expect(objs['bar'].getPhysicalPath()).result(
            ['', 'site', 'subsite', 'bar'])
        self.set_parent(objs['bar'], objs['subsite'])

        if with_trees:
            objs['subtree'] = self.providing_stub(ITopicTree)
            self.expect(objs['subsite'].objectValues()).result([
                    objs['subtree']])
        else:
            self.expect(objs['subsite'].objectValues()).result([])

        return objs

    def test_adapter_implements_interface(self):
        self.replay()
        self.assertTrue(
            ITopicRootFinder.implementedBy(DefaultTopicTreeFinder))
        verifyClass(ITopicRootFinder, DefaultTopicTreeFinder)

    def test_adapter_registered(self):
        context = self.create_dummy()
        self.replay()

        self.assertTrue(
            queryMultiAdapter((context, self.request), ITopicRootFinder),
            'ZugTopicTreeFinder adapter is not registered')

    def test_finds_next_navigation_root(self):
        objs = self.create_tree(with_trees=True)
        self.replay()

        obj = objs['foo']
        finder = getMultiAdapter((obj, self.request), ITopicRootFinder)

        self.assertEqual(finder.get_topic_root_path(), '/site')

    def test_subsite_obj_finds_subsite_with_trees(self):
        objs = self.create_tree(with_trees=True)
        self.replay()

        obj = objs['bar']
        finder = getMultiAdapter((obj, self.request), ITopicRootFinder)

        self.assertEqual(finder.get_topic_root_path(), '/site/subsite')

    def test_subsite_obj_finds_site_without_trees(self):
        objs = self.create_tree(with_trees=False)
        self.replay()

        obj = objs['bar']
        finder = getMultiAdapter((obj, self.request), ITopicRootFinder)

        self.assertEqual(finder.get_topic_root_path(), '/site')

    def test_finds_site_on_site(self):
        objs = self.create_tree(with_trees=True)
        self.replay()

        obj = objs['site']
        finder = getMultiAdapter((obj, self.request), ITopicRootFinder)

        self.assertEqual(finder.get_topic_root_path(), '/site')

    def test_fall_back_to_plone_site_root(self):
        site = self.create_dummy(getSiteManager=getSiteManager,
                                 getPhysicalPath=lambda: ('', 'site'))
        setSite(site)
        self.replay()

        finder = getMultiAdapter((self.create_dummy(), self.request),
                                 ITopicRootFinder)

        self.assertEqual(finder.get_topic_root_path(), '/site')



from Products.Five.browser import BrowserView
from ftw.topics.interfaces import ITopicBrowserLayer
from ftw.topics.testing import SIMPLELAYOUT_TOPICS_INTEGRATION_TESTING
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import login
from plone.app.testing import setRoles
from unittest2 import TestCase
from zope.component import queryMultiAdapter
from zope.interface import alsoProvides
from zope.viewlet.interfaces import IViewletManager


class TestReferencesViewletOnPage(TestCase):

    layer = SIMPLELAYOUT_TOPICS_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']

        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        login(self.portal, TEST_USER_NAME)

        self.page = self.portal.get(
            self.portal.invokeFactory('ContentPage', 'page'))

    def test_registered(self):
        # Only Registered for ITopicSupport
        self.assertFalse(self.viewlet(self.portal))
        self.assertTrue(self.viewlet(self.page))

    def test_not_available_without_references(self):
        self.assertFalse(self.viewlet().available())

    def test_available_with_references(self):
        topic1, topic2 = self.create_topic_tree("A topic", "B topic")

        self.reference_topics(topic1, topic2)
        self.assertTrue(self.viewlet().available())

        result = [
            dict(title='A topic',
                 description='',
                 url=topic1.absolute_url()),
            dict(title='B topic',
                 description='',
                 url=topic2.absolute_url()), ]

        self.assertEquals(result, self.viewlet().get_references())

    def test_references_without_view_permissions_are_not_visible(self):
        about_peter, = self.create_topic_tree("About Peter")

        self.reference_topics(about_peter)
        self.assert_references(['About Peter'])

        about_peter.manage_permission('View', roles=[], acquire=False)
        self.assert_references([])
        self.assertFalse(self.viewlet().available(),
                         "Viewlet without visible references should not be available")

    def create_topic_tree(self, *topic_titles):
        tree = self.portal.get(
            self.portal.invokeFactory('ftw.topics.TopicTree', 'tree'))

        for i, topic_title in enumerate(topic_titles):
            tree.invokeFactory('ftw.topics.Topic', 'topic%i' % i,
                               title=topic_title)
        return tree.objectValues()

    def viewlet(self, context=None):
        if context is None:
            context = self.page
        alsoProvides(context.REQUEST, ITopicBrowserLayer)

        view = BrowserView(context, context.REQUEST)
        manager_name = 'plone.belowcontent'
        manager = queryMultiAdapter(
            (context, context.REQUEST, view),
            IViewletManager,
            manager_name)
        self.failUnless(manager)

        # Set up viewlets
        manager.update()
        name = 'ftw.topics.references'
        viewlets = [v for v in manager.viewlets if v.__name__ == name]
        if len(viewlets) == 0:
            return None
        else:
            self.assertEquals(1, len(viewlets),
                              "There should only be one viewlet!")
            return viewlets[0]

    def reference_topics(self, *topics):
        self.page.Schema()['topics'].set(
            self.page, [t.UID() for t in topics])

    def assert_references(self, references):
        reference_titles = [v['title'] for v in self.viewlet().get_references()]
        self.assertEquals(references, reference_titles)



from archetypes.referencebrowserwidget import ReferenceBrowserWidget
from archetypes.schemaextender.field import ExtensionField
from archetypes.schemaextender.interfaces import IBrowserLayerAwareExtender
from archetypes.schemaextender.interfaces import IOrderableSchemaExtender
from ftw.topics import _
from ftw.topics.interfaces import ITopicBrowserLayer
from ftw.topics.interfaces import ITopicRootFinder
from ftw.topics.interfaces import ITopicSupport
from Products.Archetypes.public import ReferenceField
from zope.component import adapts
from zope.component import getMultiAdapter
from zope.component.hooks import getSite
from zope.interface import implements


class ExtendedReferenceField(ExtensionField, ReferenceField):
    """Schemaextender reference field
    """


class ATTopicSupportExtender(object):
    adapts(ITopicSupport)
    implements(IOrderableSchemaExtender, IBrowserLayerAwareExtender)

    layer = ITopicBrowserLayer

    def __init__(self, context):
        self.context = context

    def getFields(self):
        startup_directory = self.get_startup_directory()

        return [
            ExtendedReferenceField(
                name='topics',
                relationship='ftw.topics',
                allowed_types=('ftw.topics.Topic',),
                multiValued=True,
                write_permission='ftw.topics: Set Topic Reference',
                schemata='categorization',

                widget=ReferenceBrowserWidget(
                    label=_(u'label_topics', default=u'Topics'),
                    startup_directory=startup_directory,
                    )

                )]

    def get_startup_directory(self):
        site = getSite()  # CMFEditions support
        finder = getMultiAdapter((self.context, site.REQUEST),
                                 ITopicRootFinder)
        return finder.get_topic_root_path()

    def getOrder(self, schematas):
        return schematas



from ftw.topics.interfaces import ITopic
from ftw.topics.interfaces import ITopicTree
from ftw.topics.testing import TOPICS_FUNCTIONAL_TESTING
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import TEST_USER_PASSWORD
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.testing.z2 import Browser
from unittest2 import TestCase
import transaction


class TestContentTypeCreation(TestCase):

    layer = TOPICS_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']

        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        login(self.portal, TEST_USER_NAME)

        transaction.commit()

        self.browser = Browser(self.layer['app'])
        self.browser.addHeader('Authorization', 'Basic %s:%s' % (
                TEST_USER_NAME, TEST_USER_PASSWORD,))
        self.browser.handleErrors = False

    def test_create_topic_tree(self):
        self.browser.open(self.portal.portal_url())
        self.browser.getLink(id='ftw-topics-topictree').click()

        self.assertTrue(
            self.browser.url.endswith('++add++ftw.topics.TopicTree'),
            'Should be on the topic tree add view, but url is: %s' % (
                self.browser.url))

        self.browser.getControl(label='Title').value = 'Topical'
        self.browser.getControl(label='Save').click()

        self.assertTrue(
            self.browser.url.endswith('topical/view'),
            'Should be on "topical" view, but url is: %s' % (
                self.browser.url))
        self.assertIn('Topical', self.browser.contents)

        tree = self.portal.get('topical')
        self.assertTrue(ITopicTree.providedBy(tree))

    def test_create_topic(self):
        # first create a tree
        self.browser.open('http://nohost/plone/++add++ftw.topics.TopicTree')
        self.browser.getControl(label='Title').value = 'Topical'
        self.browser.getControl(label='Save').click()

        self.assertEqual(self.browser.url,
                         'http://nohost/plone/topical/view')

        factory_link = self.browser.getLink(id='ftw-topics-topic')
        self.assertTrue(
            factory_link,
            'There is no "Topic" factory link on the topic tree view.')
        factory_link.click()

        self.assertEqual(
            self.browser.url,
            'http://nohost/plone/topical/++add++ftw.topics.Topic')
        self.browser.getControl(label='Title').value = 'Manufacturing'
        self.browser.getControl(label='Save').click()

        self.assertEqual(self.browser.url,
                         'http://nohost/plone/topical/manufacturing/view')
        self.assertIn('Manufacturing', self.browser.contents)

        topic = self.portal.get('topical').get('manufacturing')
        self.assertTrue(ITopic.providedBy(topic))



from ftw.builder import Builder
from ftw.builder import create
from ftw.testbrowser import browsing
from ftw.topics.interfaces import ITopicSupport
from ftw.topics.testing import SIMPLELAYOUT_TOPICS_FUNCTIONAL_TESTING
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from Products.CMFCore.utils import getToolByName
from unittest2 import TestCase


class TestSchemaExtender(TestCase):

    layer = SIMPLELAYOUT_TOPICS_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']

        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        login(self.portal, TEST_USER_NAME)

    def test_extends_ITopicSupport(self):
        page = self.portal.get(self.portal.invokeFactory(
                               'ContentPage', 'page'))

        self.assertTrue(
            ITopicSupport.providedBy(page),
            'Expected ContentPage objects to provide ITopicSupport'
            ' but it doesnt.')

        field = page.Schema().get('topics')
        self.assertTrue(field, 'Field "topics" missing on ContentPage')

    def test_does_not_extend_others(self):
        obj = self.portal.get(self.portal.invokeFactory('File', 'file'))

        self.assertFalse(ITopicSupport.providedBy(obj),
                         'Did not expect "File" to provide "ITopicSupport"')

        field = obj.Schema().get('topics')
        self.assertFalse(
            field, '"File" object should not have "topics" field')

    @browsing
    def test_extends_ITopicSupport_with_versioning(self, browser):
        portal_repository = getToolByName(self.portal, 'portal_repository')
        portal_repository.setVersionableContentTypes(['File', ])

        file_ = create(Builder('file')
                       .with_dummy_content()
                       .providing(ITopicSupport))

        browser.login().visit(file_, view='edit')
        self.assertTrue(len(browser.css('#archetypes-fieldname-topics')),
                        'Field "topics" missing on File')

        browser.fill({'cmfeditions_save_new_version': True}).save()

        self.assertTrue(browser.url.startswith(file_.absolute_url()))

        browser.visit(file_, view='@@historyview')
        self.assertEquals(1,
                          len(browser.css('.historyRecord')),
                          'Expect one history entry')



from ftw.testbrowser import browsing
from ftw.topics.testing import EXAMPLE_CONTENT_DEFAULT_FUNCTIONAL
from ftw.topics.testing import EXAMPLE_CONTENT_SIMPLELAYOUT_FUNCTIONAL
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import TEST_USER_PASSWORD
from plone.browserlayer.layer import mark_layer
from plone.mocktestcase.dummy import Dummy
from plone.testing.z2 import Browser
from plone.uuid.interfaces import IUUID
from Products.CMFCore.utils import getToolByName
from pyquery import PyQuery
from unittest2 import TestCase
from zope.component import getMultiAdapter
import transaction


def links_to_text(pq_resultset):
    return map(lambda node: node.text.strip(), pq_resultset)


class TestDefaultTopicView(TestCase):

    layer = EXAMPLE_CONTENT_DEFAULT_FUNCTIONAL
    viewname = 'topic_view'

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        self.subsite = self.portal.get('foo').get('subsite')

        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        login(self.portal, TEST_USER_NAME)

        self.tree = self.portal.get('topics')
        self.node = self.tree.get('manufacturing')
        self.subnode = self.node.get('agile-manufacturing')
        self.topic_technology = self.tree.get('technology')

        self.subsite_tree = self.subsite.get('topics')
        self.subsite_node = self.subsite_tree.get('manufacturing')

        self.browser = Browser(self.layer['app'])
        self.browser.addHeader('Authorization', 'Basic %s:%s' % (
            TEST_USER_NAME, TEST_USER_PASSWORD,))
        self.browser.handleErrors = False

        mark_layer(None, Dummy(request=self.request))

    def test_default_topic_view(self):
        portal_types = getToolByName(self.portal, 'portal_types')
        fti = portal_types.get('ftw.topics.Topic')
        self.assertEqual(
            fti.default_view, self.viewname,
            'Expected default view to be %s on "ftw.topics.Topic"'
            ' since only the default GS profile was installed' % (
                self.viewname))

    def test_subnode_listed(self):
        self.browser.open(self.node.absolute_url() + '/' + self.viewname)
        doc = PyQuery(self.browser.contents)

        self.assertEqual(doc('h2.subelements-heading').text(), 'Topics',
                         'Expected a "Topics" heading')

        # on node (Manufacturing) should be subnode (Agile Manufacturing)
        links = doc('.subelements-listing a')
        self.assertEqual(len(links), 2,
                         'Found more or less links than expected')

        self.assertEqual(links.text(), 'Agile Manufacturing Quality')

    def test_subnode_has_no_children(self):
        self.browser.open(self.subnode.absolute_url() + '/' + self.viewname)
        doc = PyQuery(self.browser.contents)

        self.assertFalse(doc('h2.subelements-heading'),
                         'Expected no "Topics" heading')

    def test_default_section_filter_selection(self):
        self.browser.open(self.node.absolute_url() + '/' + self.viewname)
        doc = PyQuery(self.browser.contents)

        self.assertEquals(doc('.topic-filter li b').text().strip(),
                          'Plone site',
                          'Expected section "Plone site" to be ' +
                          'selected by default.')

        filter_links = links_to_text(doc('.topic-filter li a'))
        self.assertIn('Sub Site', filter_links,
                      'Missing awailable section filter "Sub Site"')

        reference_links = links_to_text(doc('.topic-reference-listings a'))
        self.assertIn(
            'Manufacturing processes', reference_links,
            'Link "Manufacturing processes" should be shown')

        self.assertNotIn(
            'Theories', reference_links,
            'Link "Theories" should not be shown, it is in'
            ' the section "Sub Site", not "Plone site"')

    def test_other_section_filter(self):
        self.browser.open(self.node.absolute_url() + '/' + self.viewname)
        self.browser.getLink('Sub Site').click()
        doc = PyQuery(self.browser.contents)

        self.assertEquals(doc('.topic-filter li b').text().strip(),
                          'Sub Site',
                          'Expected section "Sub Site" to be ' +
                          'selected by default.')

        filter_links = links_to_text(doc('.topic-filter li a'))
        self.assertIn('Plone site', filter_links,
                      'Missing awailable section filter "Plone site"')

        reference_links = links_to_text(doc('.topic-reference-listings a'))
        self.assertNotIn(
            'Manufacturing processes', reference_links,
            'Link "Manufacturing processes" should not be shown, it is in'
            ' the section "Sub Site", not "Plone site"')

        self.assertIn(
            'Theories', reference_links,
            'Link "Theories" should be shown')

    def test_SUBSITE_default_section_filter_selection(self):
        self.browser.open(self.subsite_node.absolute_url() + '/' +
                          self.viewname)
        doc = PyQuery(self.browser.contents)

        self.assertEquals(doc('.topic-filter li b').text().strip(),
                          'Sub Site',
                          'Expected section "Sub Site" to be ' +
                          'selected by default.')

        filter_links = links_to_text(doc('.topic-filter li a'))
        self.assertIn('Plone site', filter_links,
                      'Missing awailable section filter "Plone site"')

        reference_links = links_to_text(doc('.topic-reference-listings a'))
        self.assertNotIn(
            'Manufacturing processes', reference_links,
            'Link "Manufacturing processes" should not be shown, it is in'
            ' the section "Sub Site", not "Plone site"')

        self.assertIn(
            'Theories', reference_links,
            'Link "Theories" should be shown')

    def test_sections_are_always_shown_when_there_are_subsites_and_brefs(self):
        self.browser.open(self.topic_technology.absolute_url() + '/' +
                          self.viewname)
        doc = PyQuery(self.browser.contents)

        self.assertEquals(
            doc('.topic-filter li').text(), 'Plone site',
            'Only Plone site should be shown as section, because there'
            ' are other sections (subsites) - even when there is only one'
            ' section shown.')

    def test_sections_are_not_shown_when_there_are_subsites_but_no_brefs(self):
        self.topic_quality = self.node.get('quality')
        self.browser.open(self.topic_quality.absolute_url() + '/' +
                          self.viewname)
        doc = PyQuery(self.browser.contents)

        self.assertEquals(doc('.topic-filter li'), [],
                          'Expect no section, because there is no content')

    def test_sections_are_shown_when_other_sections_have_brefs(self):
        obj = self.subsite_tree.get('technology')
        self.browser.open(obj.absolute_url() + '/' + self.viewname)
        doc = PyQuery(self.browser.contents)

        self.assertEquals(
            ['Plone site', 'Sub Site'],
            map(lambda node: PyQuery(node).text(), doc('.topic-filter li')),
            'Section filter should be visible because other sections have'
            ' backreferences.')

    def test_no_section_are_shown_when_there_are_no_subsites(self):
        # delete all "subsites", so that we have only one "section",
        # which is the site.
        self.portal.manage_delObjects(['foo', 'empty-subsite'])
        transaction.commit()

        self.browser.open(self.topic_technology.absolute_url() + '/' +
                          self.viewname)
        doc = PyQuery(self.browser.contents)

        self.assertEquals(
            0, len(doc('.topic-filter')),
            'Sections should not be shown in a non-subsite setup.')

    def test_backreferences_without_view_permissions_are_not_visible(self):
        folder = self.portal.get('foo')
        folder.Schema()['topics'].set(folder, self.subnode.UID())

        self.topicview = getMultiAdapter((self.subnode, self.request),
                                         name='topic_view')

        self.assert_references(['Foo'])

        folder.manage_permission('View', roles=[], acquire=False)

        self.assert_references([])

    def assert_references(self, references):
        self.topicview()
        reference_titles = [v['title'] for v in self.topicview.objects]
        self.assertEquals(references, reference_titles)

    @browsing
    def test_settings_can_hide_backreferences(self, browser):
        browser.login(username=SITE_OWNER_NAME, password=SITE_OWNER_PASSWORD)

        browser.visit(self.topic_technology)
        self.assertTrue(browser.css('.referenceRepresentationListing'))

        browser.visit(self.topic_technology, view='edit')
        browser.fill({'Show backreferences': False}).submit()

        self.assertFalse(browser.css('.referenceRepresentationListing'))


class TestSimplelayoutTopicView(TestDefaultTopicView):

    layer = EXAMPLE_CONTENT_SIMPLELAYOUT_FUNCTIONAL
    viewname = 'simplelayout'

    def setUp(self):
        super(TestSimplelayoutTopicView, self).setUp()

        self.page = self.portal.get(self.portal.invokeFactory(
            'ContentPage', 'page', title="Page 1"))

        self.folder = self.portal.get(
            self.portal.invokeFactory('Folder', 'folder', title="Folder"))
        transaction.commit()

    def test_representation_contentpage(self):
        self.page.Schema()['topics'].set(self.page, IUUID(self.subnode))
        self.page.reindexObject()
        transaction.commit()

        self.browser.open(self.subnode.absolute_url())
        doc = PyQuery(self.browser.contents)

        self.assertEquals(
            len(doc('.referenceRepresentationListing ul li a')), 1,
            'Found more or less links than expected')

        self.assertEquals(
            len(doc('.referenceRepresentationTitle')), 1,
            'Found more or less links than expected')

    def test_representation_default(self):
        self.folder.Schema()['topics'].set(self.folder, IUUID(self.subnode))
        self.folder.reindexObject()
        transaction.commit()

        self.browser.open(self.subnode.absolute_url())
        doc = PyQuery(self.browser.contents)

        self.assertEquals(
            len(doc('.referenceRepresentationListing ul li a')), 1,
            'Found more or less links than expected')

        self.assertEquals(
            len(doc('.referenceRepresentationTitle')), 1,
            'Found more or less links than expected')
